/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.repository;

import java.util.List;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import se.vgregion.verticalprio.entity.AbstractHirarkiskKod;

@Repository
public class JpaGenerisktHierarkisktKodRepository<T extends AbstractHirarkiskKod> extends
        JpaGenerisktKodRepository<T> implements GenerisktHierarkisktKodRepository<T> {

	public JpaGenerisktHierarkisktKodRepository() {
		super((Class<T>) AbstractHirarkiskKod.class);
	}

	public JpaGenerisktHierarkisktKodRepository(Class<T> klass) {
		super(klass);
	}

	@Override
	@Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
	public List<T> getTreeRoots() {
		List<T> result = query("select o from @ o where o.parentId is null order by LOWER(o.kod)", null);
		initChildren(result);
		return result;
	}

	@SuppressWarnings("unchecked")
	private void initChildren(List<T> items) {
		for (T item : items) {
			initChildren(item.getChildren());
		}
	}

	/* @Transactional(propagation = Propagation.SUPPORTS, readOnly = true) private void
	 * populateChildren(Collection<T> parents) { for (T child : parents) { populateChildren(child); } }
	 * 
	 * @Transactional(propagation = Propagation.SUPPORTS, readOnly = true) private void populateChildren(T parent)
	 * { String qt = "select o from @ o where o.parent = ?1"; parent.getChildren().addAll(query(qt, parent));
	 * populateChildren(parent.getChildren()); } */

}
