/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.repository;

import java.util.Collection;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import se.vgregion.dao.domain.patterns.repository.db.jpa.DefaultJpaRepository;
import se.vgregion.verticalprio.entity.SektorRaad;

@Repository
public class JpaSektorRaadRepository extends DefaultJpaRepository<SektorRaad> implements SektorRaadRepository {

    @Override
    @Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
    public List<SektorRaad> getTreeRoots() {
        List<SektorRaad> result = query("select o from @ o where o.parent is null order by LOWER(o.code) DESC");
        populateChildren(result);
        return result;
    }

    @Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
    private void populateChildren(Collection<SektorRaad> parents) {
        for (SektorRaad child : parents) {
            populateChildren(child);
        }
    }

    @Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
    private void populateChildren(SektorRaad parent) {
        String qt = "select o from @ o where o.parent = ?1";
        parent.getChildren().addAll(query(qt, parent));
        populateChildren(parent.getChildren());
    }

    @SuppressWarnings("unchecked")
    @Transactional(propagation = Propagation.SUPPORTS, readOnly = true)
    private List<SektorRaad> query(String qt, Object... values) {
        qt = qt.replace("@", SektorRaad.class.getSimpleName());
        Query query = entityManager.createQuery(qt);
        int i = 1;
        for (Object value : values) {
            query.setParameter(i++, value);
        }
        return query.getResultList();
    }

}
