/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.repository.finding;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Claes Lundahl, vgrid=clalu4
 * 
 */
public class CoalesceingOrderByPath extends OrderByPath {

    private static final long serialVersionUID = 1L;

    private static int count = 0;

    private static int currentCount = count++;

    private List<OrderByPath> orders;

    public CoalesceingOrderByPath(OrderByPath... paths) {
        orders = Arrays.asList(paths);
    }

    /**
     * @inheritDoc
     */
    @Override
    public List<String> toJpqlJoinPart(String startAlias) {
        List<String> parts = new ArrayList<String>();
        for (OrderByPath obp : orders) {
            parts.addAll(obp.toJpqlJoinPart(startAlias));
        }
        return parts;
    }

    /**
     * @inheritDoc
     */
    @Override
    public String toJpqlSelectPart(String alias) {
        List<String> parts = new ArrayList<String>();
        for (OrderByPath obp : orders) {
            parts.add(obp.toJpqlSelectPart(alias));
        }
        return "coalesce(" + (JpqlMatchBuilder.toString(parts, ", ")) + ") as sorting" + currentCount;
    }

    /**
     * @inheritDoc
     */
    @Override
    public String toJpqlOrderByPart(String alias) {
        String result = "sorting" + currentCount;
        result += super.isAscending() ? " asc" : " desc";
        return result;
    }

}
