/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.verticalprio.repository.finding;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @author Claes Lundahl, vgrid=clalu4
 * 
 */
public class OrderByPath implements Serializable {

    private static final long serialVersionUID = 1L;

    private static long totalCount = 0;

    private final long id = totalCount++;

    private boolean ascending = true;

    private String[] path;

    public OrderByPath() {

    }

    public OrderByPath(String path) {
        setPath(path);
    }

    public OrderByPath(String path, boolean ascending) {
        setAscending(ascending);

    }

    public void setAscending(boolean ascending) {
        this.ascending = ascending;
    }

    public boolean isAscending() {
        return ascending;
    }

    public void setPath(String path) {
        this.path = path.split(Pattern.quote("/"));

    }

    public String getPath() {
        return JpqlMatchBuilder.toString(Arrays.asList(path), "/");
    }

    /**
     * @inheritDoc
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (!(obj instanceof OrderByPath)) {
            return false;
        }
        OrderByPath obp = (OrderByPath) obj;
        if (!equals(path, obp.path)) {
            return false;
        }
        if (ascending != obp.ascending) {
            return false;
        }
        return true;
    }

    private boolean equals(Object o1, Object o2) {
        if (o1 == o2) {
            return true;
        }
        if (o1 == null || o2 == null) {
            return false;
        }
        return o1.equals(o2);
    }

    public List<String> toJpqlJoinPart(String startAlias) {
        List<String> remainingPath = new ArrayList<String>(Arrays.asList(path));
        List<String> result = new ArrayList<String>();
        toJpqlJoinPart(startAlias, remainingPath, 0, result);
        return result;
    }

    private void toJpqlJoinPart(String startAlias, List<String> remainingPath, int count, List<String> result) {

        if (remainingPath.size() <= 1) {
            return;
        }

        StringBuilder sb = new StringBuilder();
        sb.append(startAlias + "." + remainingPath.remove(0));
        String nextAlias = "ob" + getAliasCount() + "c" + count;
        sb.append(" " + nextAlias);
        result.add(sb.toString());

        toJpqlJoinPart(nextAlias, remainingPath, count + 1, result);

    }

    public static List<String> toJpqlJoinParts(Iterable<OrderByPath> paths, String alias) {
        List<String> result = new ArrayList<String>();
        for (OrderByPath path : paths) {
            result.addAll(path.toJpqlJoinPart(alias));
        }
        return result;
    }

    public String toJpqlSelectPart(String alias) {
        if (path.length == 1) {
            return alias + "." + path[0];
        }
        String result = "ob" + getAliasCount() + "c" + (path.length - 2);
        result += "." + path[path.length - 1];
        return result;
    }

    public static List<String> toJpqlSelectParts(Iterable<OrderByPath> paths, String alias) {
        List<String> result = new ArrayList<String>();
        for (OrderByPath path : paths) {
            result.add(path.toJpqlSelectPart(alias));
        }
        return result;
    }

    public String toJpqlOrderByPart(String alias) {
        String result = toJpqlSelectPart(alias);
        result += ascending ? " asc" : " desc";
        return result;
    }

    public long getAliasCount() {
        return id;
    }

    public static List<String> toJpqlOrderByParts(Iterable<OrderByPath> paths, String alias) {
        List<String> result = new ArrayList<String>();
        for (OrderByPath path : paths) {
            result.add(path.toJpqlOrderByPart(alias));
        }
        return result;
    }

}
